﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.VariableInteractionNetworks {
  public partial class CreateTargetVariationExperimentDialog : Form {
    public Experiment Experiment { get; private set; }
    private int repetitions;

    public CreateTargetVariationExperimentDialog() : this(null) { }
    public CreateTargetVariationExperimentDialog(IAlgorithm alg) {
      InitializeComponent();
      repetitions = (int)repetitionsNumericUpDown.Value;

      Experiment = null;
      okButton.Enabled = alg != null;
    }

    private void repetitionsNumericUpDown_Validated(object sender, EventArgs e) {
      if (repetitionsNumericUpDown.Text == string.Empty)
        repetitionsNumericUpDown.Text = repetitionsNumericUpDown.Value.ToString();
      repetitions = (int)repetitionsNumericUpDown.Value;
    }

    public Experiment CreateVariableCombinations(IAlgorithm algorithm) {
      var experiment = new Experiment("Target Variation Experiment");

      if (algorithm == null)
        throw new ArgumentNullException("The algorithm parameter must be a valid IAlgorithm instance.");

      var regressionProblem = algorithm.Problem as IRegressionProblem;
      if (regressionProblem == null)
        throw new ArgumentNullException("The algorithm does not contain a valid problem instance.");

      var problemData = regressionProblem.ProblemData;
      var variables = problemData.Dataset.DoubleVariables.ToList();

      for (int i = 0; i < variables.Count; i++) {
        var alg = (IAlgorithm)algorithm.Clone();
        var pd = ((IRegressionProblem)alg.Problem).ProblemData;

        for (int j = 0; j < variables.Count; ++j)
          pd.InputVariables.SetItemCheckedState(pd.InputVariables[j], true);

        pd.InputVariables.SetItemCheckedState(pd.InputVariables[i], false);
        pd.TargetVariable = pd.InputVariables[i].Value;

        alg.Name += "{Target = " + pd.TargetVariable + "}";

        var batchrun = new BatchRun(string.Format("Batchrun {0}", i + 1)) {
          Repetitions = repetitions,
          Optimizer = alg
        };

        experiment.Optimizers.Add(batchrun);
      }
      return experiment;
    }

    private void okButton_Click(object sender, EventArgs e) {
      IContentView activeView = MainFormManager.MainForm.ActiveView as IContentView;
      if ((activeView != null) && (activeView.Content != null) && (activeView.Content is IOptimizer) && !activeView.Locked) {
        Experiment = CreateVariableCombinations((IAlgorithm)activeView.Content);
        DialogResult = DialogResult.OK;
        Close();
      }
    }
  }
}
